module Eco
  module API
    module Common
      # Helpers for dynammic generation of classes based on a hierarchical model
      # @attr_reader model [Hash, nil] the `model` of the current `class`
      module ClassHierarchy
        include Eco::API::Common::ClassHelpers

        attr_reader :model

        # @param value [Hash, Enumerable, String, Symbol, nil] unparsed model to be assigned to the `class`
        def model=(value)
          @model = parse_model(value)
        end

        # @return [Array<String>] the `keys` of the current class' `model`
        def model_attrs
          (model && model.keys) || []
        end

        # Thanks to this step the format on the declaration of the model is flexible
        # @param model [Hash, Enumerable, String, Symbol, nil]
        # @return [Hash, nil] where keys are `Symbol` s
        def parse_model(model)
          case model
          when String
            return parse_model(model.to_sym)
          when Symbol
            return {model => nil}
          when Hash
            return model.each_with_object({}) do |(k,v), hash|
              hash[k.to_sym] = v
            end
          when Enumerable
            return model.each_with_object({}) do |sub, hash|
              hash.merge!(parse_model(sub))
            end
          when NilClass
            return nil
          else
            raise "Incorrect model declaration, allowed String, Symbol, Hash and Enumerable. Given: #{model.class}"
          end
        end

      end
    end
  end
end
