module Eco
  module API
    class Session
      class Config
        class People < BaseConfig

          def cache=(file)
            self["cache"] = file
          end

          def cache
            self["cache"] ||= "cache/people.json"
          end

          def partial_cache
            if file = self.cache
              File.join(File.dirname(file),
                File.basename(file, File.extname(file)) + "_partial" + File.extname(file)
              )
            end
          end

          # specifies if people search should be strict or not
          def strict_search=(value)
            self["strict_search"] = !!value
          end

          def strict_search?
            self["strict_search"]
          end

          # api queried logs
          def requests_folder=(folder)
            self["requests_folder"] = folder
          end

          def requests_folder
            self["requests_folder"] ||= "requests"
          end

          # people to exclude from update feeds
          def discarded_file=(file)
            self["discarded_file"] = file
          end

          def discarded_file
            self["discarded_file"]
          end

          # with given a person what is the criteria of exclusion
          def api_excluded
            return self["api_excluded"] unless block_given?
            self["api_excluded"] = Proc.new
          end

          # internal-external fields map
          def fields_mapper=(file)
            self["fields_mapper"] = file
          end

          def fields_mapper
            self["fields_mapper"]
          end

          # person model
          def default_usergroup=(value)
            self["default_usergroup"] = value
          end

          def default_usergroup
            self["default_usergroup"]
          end

          def default_usergroup?
            !!self["default_usergroup"]
          end

          def default_schema=(name)
            self["default_schema"] = name
          end

          def default_schema
            self["default_schema"]
          end

          def default_schema?
            !!self["default_schema"]
          end
          
          # @return [Hash] with defined pairs format `key` and Person parsers.
          def parsers
            self["parsers"] ||= {}
          end

          # The person parser/serializer for a given format (default `csv`).
          # @param format [Symbol] the format this parser/serializer recognizes.
          # @return [Eco::API::Common::People::PersonParser] parser/serializer for the defined `format`.
          def parser(format: :csv)
            prs = parsers[format] ||= Eco::API::Common::People::PersonParser.new
            if block_given?
              yield(prs)
              self
            else
              prs
            end
          end

        end
      end
    end
  end
end
