module Eco
  module API
    class UseCases
      # Core class of UseCases. It basically defines and manages allowed `types`
      class BaseCase

        class InvalidType < StandardError
          def initialize(msg = nil, type:, types:)
            msg ||= "Invalid type."
            msg = "Given type '#{type}'. Valid types: #{types}"
            super(msg)
          end
        end

        extend Eco::API::Common::ClassHelpers

        @types = [:import, :filter, :transform, :sync, :error_handler, :export, :other]

        class << self
          attr_reader :types

          def valid_type?(type)
            types.include?(type)
          end

          def validate_type(type)
            raise InvalidType.new(type: type, types: types) unless valid_type?(type)
          end
        end

      end
    end
  end
end
