class Eco::API::UseCases::DefaultCases::CleanUnknownTags < Eco::API::Common::Loaders::UseCase
  name "clean-unknown-tags"
  type :transform

  REGISTER_TAGS = [
    "EVENT", "INJURY", "RISK", "CONTRACTOR", "PERMIT",
    "AUDIT", "JSEA",
    "TRAINING", "INDUCTION",
    "MEETING",  "PPE",  "CHEMICAL",
    "PLANT", "ASSET",
    "POLICY", "IDEA", "REPORTS"
  ]

  attr_reader :session, :options

  def main(people, session, options, usecase)
    @session = session; @options = options

    update   = session.new_job("main", "update", :update, usecase)
    people.each do |person|
      unknown_tags = person.filter_tags.select {|tag| !tag?(tag)}
      person.filter_tags -= unknown_tags
      update.add(person)
    end
  end

  private

  def tag?(value)
    tagtree.tag?(value) || REGISTER_TAGS.any? {|reg| value == reg}
  end

  def tagtree
    @tagtree ||= ASSETS.config.tagtree
  end

end
