module Eco
  class CLI
    class Config
      class Input

        attr_reader :core_config
        attr_reader :default_option

        def initialize(core_config:, default_option: nil)
          @core_config    = core_config
          @default_option = default_option
          @callbacks = {}
        end

        def default_option?
          !!default_option
        end

        def define(option: nil, &block)
          option ||= default_option
          raise "Missing option to identify the input (no default defined)" unless !!option
          raise "Missing block to define the input obtention process" unless block
          @callbacks[option] = block
        end

        def get(io:, option: nil)
          unless io && io.is_a?(Eco::API::UseCases::BaseIO)
            raise "You need to provide Eco::API::UseCases::BaseIO object. Given: #{io.class}"
          end

          option ||= default_option
          raise "Missing option to identify the input (no default defined)" unless !!option
          callback = @callbacks[option] || @callbacks[default_option]
          callback.call(io.session, option, io.options)
        end

      end
    end
  end
end
