module Eco
  module Data
    module Files
      DEFAULT_TIMESTAMP_PATTERN = '%Y-%m-%dT%H%M%S'

      class << self
        def included(base)
          base.send(:include, InstanceMethods)
          base.extend(ClassMethods)
        end
      end
      
      module InstanceMethods

      end

      module ClassMethods
        def create_directory(path, includes_file: false)
          Directory.create(path, includes_file: includes_file)
        end

        def split(path)
          dir_path, file = File.split(path)
          dir_path = dir_path.gsub(File::SEPARATOR, File::ALT_SEPARATOR || File::SEPARATOR)
          dir_path.split(File::ALT_SEPARATOR || File::SEPARATOR).push(file)
        end

        def file_name(fullname)
          File.basename(fullname)
        end

        def file_basename(fullname)
          File.basename(fullname, File.extname(fullname))
        end

        def file_path(fullname)
          File.dirname(fullname)
        end

        def file_fullpath(fullname)
          file_path(File.expand_path(fullname))
        end

        def file_exists?(file)
          return false if !file
          return File.exists?(file) || File.exists?(File.expand_path(file))
        end

        def dir_exists?(path)
          Dir.exist?(path) || Dir.exist?(File.expand_path(path))
        end

        def timestamp(timestamp_pattern = DEFAULT_TIMESTAMP_PATTERN)
          Time.now.strftime(timestamp_pattern)
        end

        def timestamp_file(filename, timestamp_pattern = DEFAULT_TIMESTAMP_PATTERN)
          file_pattern = Eco::Data::Files::FilePattern.new(filename)
          file_pattern.resolve(start: timestamp(timestamp_pattern) + '_')
        end

        def copy_file(source_file, dest_file, time_stamp: false)
          dest_file = timestamp_file(dest_file) if time_stamp
          File.write(dest_file, File.read(source_file))
        end

        def file_empty?(path)
          return true if !File.file?(path)
          File.zero?(path)
        end

        def has_bom?(path)
          return false if !path || file_empty?(path)
          File.open(path, "rb") do |f|
            bytes = f.read(3)
            return bytes.unpack("C*") == [239, 187, 191]
          end
        end

        def encoding(path)
          has_bom?(path) ? "bom" : "utf-8"
        end

        def script_subfolder
          basename = File.basename($0, File.extname($0))
          path = File.dirname($0)
          File.join(path, basename)
        end
      end

      class << self
        include Files::ClassMethods
      end

    end
  end
end
