module Eco
  module Language
    class HashTransformModifier < Eco::Language::Models::Modifier

      FLAT_MODE         = [:flat, :non_recursivre]
      RECURSIVE_MODE    = [:recurse, :deep, :recursive, :r]

      COPY_MODE         = [:copy, :clone, :c]
      BANG_MODE         = [:!, :bang]

      COLON_MODE        = [:colon, :colon_to_key, :only_colon]
      ALL_TO_SYM        = [:all_to_sym, :all]

      STRINGIFY_MODE    = [:stringify, :string, :to_s]
      SYMBOLIZE_MODE    = [:symbolize, :sym, :to_sym]

      KEYS_MODE         = [:keys]
      NO_KEYS_MODE      = [:no_keys]

      NO_VALUES_MODE    = [:no_values]
      VALUES_MODE       = [:values]

      def reset_recursive
        self < self.mode - (RECURSIVE_MODE | FLAT_MODE)
      end

      def reset_bang
        self < self.mode - (BANG_MODE | COPY_MODE)
      end

      def reset_colon_mode
        self < self.mode - (COLON_MODE | ALL_TO_SYM)
      end

      def recurse
        self.push(:recurse)
      end

      def non_recurse
        self.push(:flat)
      end

      def copy
        self.push(:copy)
      end

      def bang
        self.push(:bang)
      end

      def colon
        self.push(:colon)
      end

      def all
        self.push(:all)
      end

      def symbolize
        self.push(:symbolize)
      end

      def stringify
        self.push(:stringify)
      end

      def keys
        self.push(:keys)
      end

      def values
        self.push(:values)
      end

      def recurse?
        mode.any? { |m| RECURSIVE_MODE.include?(m) }
      end

      def clone?
        mode.all? { |m| !BANG_MODE.include?(m) }
      end

      def colon?
        mode.any? { |m| COLON_MODE.include?(m) }
      end

      def symbolize?
        mode.any? { |m| SYMBOLIZE_MODE.include?(m) }
      end

      def keys?
        mode.any? { |m| KEYS_MODE.include?(m) }
      end

      def values?
        mode.any? { |m| VALUES_MODE.include?(m) }
      end

      protected

      def resolve_mode
        modifiers = self.to_a
        modifiers = resolve(modifiers, RECURSIVE_MODE | FLAT_MODE)
        modifiers = resolve(modifiers, BANG_MODE | COPY_MODE)
        modifiers = resolve(modifiers, COLON_MODE | ALL_TO_SYM)
        modifiers = resolve(modifiers, STRINGIFY_MODE | SYMBOLIZE_MODE)
        modifiers = resolve(modifiers, KEYS_MODE | NO_KEYS_MODE)
        modifiers = resolve(modifiers, VALUES_MODE | NO_VALUES_MODE)
        modifiers
      end

    end
  end
end
