module Eco
  module Data
    module FuzzyMatch
      class Result < Struct.new(:match, :value, :needle_value, :dice, :levenshtein, :jaro_winkler, :ngrams, :words_ngrams, :chars_position)
        ALL_METHODS = [:dice, :levenshtein, :jaro_winkler, :ngrams, :words_ngrams, :chars_position]

        attr_accessor :pivot

        def dice; super&.round(3); end
        def levenshtein; super&.round(3); end
        def jaro_winkler; super&.round(3); end
        def ngrams; super&.round(3); end
        def words_ngrams; super&.round(3); end
        def chars_position; super&.round(3); end

        #Shortcuts
        def lev; levenshtein; end
        def jaro; jaro_winkler; end
        def wngrams; words_ngrams; end
        def pos; chars_position; end

        def average
          values = [dice, levenshtein, jaro_winkler, ngrams, words_ngrams, chars_position]
          (values.inject(0.0, :+) / values.length).round(3)
        end

        # TODO: print in the order of `order`
        def print
          msg  = "(Dice: #{dice}) (Lev Dst: #{levenshtein}) "
          msg << "(Jaro: #{jaro_winkler}) "
          msg << "(Ngram: #{ngrams}) (WNgrams: #{words_ngrams}) "
          msg << "(C Pos: #{chars_position}) "
          msg << "(Avg: #{average}) "
          msg << "'#{value}'"
        end

        def all_threshold?(methods = order, threshold = 0.15)
          return true unless threshold
          [methods].flatten.compact.all? {|method| threshold?(method, threshold)}
        end

        def any_threshold?(methods = order, threshold = 0.15)
          return true unless threshold
          [methods].flatten.compact.any? {|method| threshold?(method, threshold)}
        end

        def threshold?(method = :dice, threshold = 0.15)
          raise "Uknown method '#{method}'" unless self.respond_to?(method)
          self.send(method) >= threshold
        end

        def order=(values)
          @order = [values].flatten.compact.tap do |o|
            o << [:words_ngrams, :dice] if o.empty?
          end
        end

        def order
          @order ||= [:words_ngrams, :dice]
        end

        def <=>(result)
          compare(result)
        end

        def values_at(*keys)
          keys.map do |key|
            self.send(key) if self.respond_to?(key)
          end
        end

        private

        def compare(other, order: self.order)
          return 0  unless method = order.first
          raise "Uknown method '#{method}'"  unless self.respond_to?(method) && other.respond_to?(method)
          return -1 if self.send(method) > other.send(method)
          return  1 if self.send(method) < other.send(method)
          compare(other, order: order[1..-1])
        end


      end

    end
  end
end
